@@ -1,6 +1,6 @@
 import subprocess
 from abc import ABCMeta, abstractmethod
-from contextlib import AsyncExitStack, asynccontextmanager
+from contextlib import asynccontextmanager, nullcontext
 from dataclasses import dataclass
 from os import environ
 from pathlib import Path
@@ -10,6 +10,7 @@ from typing import (
     AsyncIterator,
     Callable,
     Iterable,
+    Literal,
     Optional,
     cast,
 )
@@ -22,18 +23,18 @@ from .utils import RunArg, Some, T, async_cached, intercalate, unsome
 
 class OS(metaclass=ABCMeta):
     @abstractmethod
-    def switch_(self, *, linux: Callable[["Linux"], T], macos: T) -> T:
+    def switch_(self, *, linux: Callable[["Linux"], T], macos: Callable[[], T]) -> T:
         pass
 
     def switch(self, *, linux: T, macos: T) -> T:
-        return self.switch_(linux=lambda _: linux, macos=macos)
+        return self.switch_(linux=lambda _: linux, macos=lambda: macos)
 
 
 @dataclass
 class Linux(OS):
     distribution: str
 
-    def switch_(self, *, linux: Callable[["Linux"], T], macos: T) -> T:
+    def switch_(self, *, linux: Callable[["Linux"], T], macos: Callable[[], T]) -> T:
         return linux(self)
 
     RELEASE_FILE = "/etc/os-release"
@@ -51,8 +52,11 @@ class Linux(OS):
 
 @dataclass
 class MacOS(OS):
-    def switch_(self, *, linux: Callable[["Linux"], T], macos: T) -> T:
-        return macos
+    def switch_(self, *, linux: Callable[["Linux"], T], macos: Callable[[], T]) -> T:
+        return macos()
+
+
+Architecture = Literal["x86_64", "aarch64"]
 
 
 @dataclass
@@ -170,8 +174,14 @@ class Driver(metaclass=ABCMeta):
         return [Path(result) for result in output.split("\0") if result]
 
     @asynccontextmanager
-    async def tempfile(self) -> AsyncIterator[Path]:
-        path = Path(await self.run_output("mktemp"))
+    async def tempfile(
+        self, kind: Optional[Literal["directory"]] = None
+    ) -> AsyncIterator[Path]:
+        args = []
+        if kind == "directory":
+            args.append("-d")
+
+        path = Path(await self.run_output("mktemp", *args))
         try:
             yield path
         finally:
@@ -180,6 +190,11 @@ class Driver(metaclass=ABCMeta):
     async def makedirs(self, path: Path) -> None:
         await self.run("mkdir", "-p", path)
 
+    async def copy(self, source: Path, target: Path) -> None:
+        await self.makedirs(target.parent)
+        await self.rm(target)
+        await self.run("cp", "-R", "-f", source, target)
+
     async def rm(self, path: Path) -> None:
         await self.run("rm", "-r", "-f", path)
 
@@ -213,7 +228,18 @@ class Driver(metaclass=ABCMeta):
         elif os_type == "Darwin":
             return MacOS()
         else:
-            raise ValueError(f"Unsupported OS type {os_type}.")
+            raise ValueError(f"Unsupported OS type {os_type}.")  # pragma: no cover
+
+    @async_cached
+    async def architecture(self) -> Architecture:
+        result = await self.with_root(False).run_output("uname", "-m")
+        if result == "x86_64":
+            return "x86_64"
+        elif result in {"arm64", "aarch64"}:
+            # No ARM runners on CI
+            return "aarch64"  # pragma: no cover
+        else:
+            raise ValueError(f"Unsupported architecture {result}.")  # pragma: no cover
 
 
 class SubprocessDriver(Driver, metaclass=ABCMeta):
@@ -232,7 +258,7 @@ class SubprocessDriver(Driver, metaclass=ABCMeta):
         if capture_output and show_output:
             raise ValueError(
                 "Cannot use capture_output and show_output at the same time."
-            )
+            )  # pragma: no cover
 
         command = self.prepare_command(args)
 
@@ -246,8 +272,7 @@ class SubprocessDriver(Driver, metaclass=ABCMeta):
         if show_output:
             cm = parallel_map_pause()
         else:
-            # Should be nullcontext, but it is not async-enabled in Python 3.9.
-            cm = AsyncExitStack()
+            cm = nullcontext()
         async with cm:
             result = await run_process(
                 command,
