@@ -1,30 +1,30 @@
-from typing import Any, Tuple, Type
+from typing import Any, Type
 
 from ..driver import Driver
 from ..state import DB
 from .base import Package
+from .brew_repo import BrewRepo
 from .clone import Clone
 from .github import GitHubPackage
 from .links import Links
 from .npm import NpmPackage
-from .pip import PipPackage
 from .pipx import PipxPackage
 from .shell import Shell
 from .system import SystemPackage
 from .url import URLPackage
 from .yum_repo import YumRepo
 
-TYPES: list[Tuple[str, Type[Package]]] = [
-    ("name", SystemPackage),
+TYPES: list[tuple[str, Type[Package]]] = [
+    ("system", SystemPackage),
     ("repo", GitHubPackage),
     ("url", URLPackage),
     ("clone", Clone),
     ("npm", NpmPackage),
-    ("pip", PipPackage),
     ("pipx", PipxPackage),
     ("shell", Shell),
     ("links", Links),
     ("yum_name", YumRepo),
+    ("brew_tap", BrewRepo),
 ]
 
 
