@@ -1,18 +1,26 @@
-import shlex
 from abc import ABC, abstractmethod
 from pathlib import Path
-from typing import Iterable, Union
+from typing import Iterable
+
+from pydantic import Field
 
 from ..configparser import DesktopEntry
-from ..utils import async_cached
+from ..extractor import get_extractor
+from ..tracker import Tracker
+from ..utils import allow_singular_none, async_cached
 from .manual import ManualPackage
-from .tracked import Tracker
+
+ICON_EXTENSIONS = ["svg", "png"]
+
+FONT_EXTENSIONS = ["ttf", "otf"]
 
 
 class ArchivePackage(ManualPackage, ABC):
-    raw: Union[bool, str] = False
+    raw: bool | str = False
     raw_executable: bool = False
-    strip: int = 0
+
+    binary_paths: list[str] = Field(default_factory=list, alias="binary_path")
+    binary_paths_val = allow_singular_none("binary_paths")
 
     @abstractmethod
     async def archive_url(self) -> str:
@@ -26,43 +34,6 @@ class ArchivePackage(ManualPackage, ABC):
     async def package_directory(self) -> Path:
         return (await self.local()) / "mybox" / self.pathname
 
-    @async_cached
-    async def tar(self) -> str:
-        return await self.driver.find_executable("gtar", "tar")
-
-    async def untar(self, source: Path, *extra: str) -> None:
-        await self.driver.run(
-            await self.tar(),
-            "-x",
-            "--strip",
-            str(self.strip),
-            "-C",
-            await self.package_directory(),
-            *extra,
-            "-f",
-            source,
-        )
-
-    async def unzip(self, source: Path) -> None:
-        if self.strip > 0:
-            raise NotImplementedError("Strip is not supported for unzip.")
-        await self.driver.run(
-            "unzip", "-o", "-qq", source, "-d", await self.package_directory()
-        )
-
-    async def appimage(self, source: Path) -> None:
-        async with self.driver.tempfile() as target:
-            await self.driver.run("cp", source, target)
-            await self.driver.make_executable(target)
-
-            cmd = " && ".join(
-                [
-                    shlex.join(["cd", str(await self.package_directory())]),
-                    shlex.join([str(target), "--appimage-extract"]),
-                ]
-            )
-            await self.driver.run("sh", "-c", cmd)
-
     async def extract(self, url: str, source: Path) -> None:
         if self.raw:
             if isinstance(self.raw, str):
@@ -70,23 +41,15 @@ class ArchivePackage(ManualPackage, ABC):
             else:
                 filename = url.rsplit("/", 1)[-1]
             target = await self.package_directory() / filename
-            await self.driver.run("cp", source, target)
+            await self.driver.copy(source, target)
             if self.raw_executable:
                 await self.driver.make_executable(target)
-        elif url.endswith(".tar"):
-            await self.untar(source)
-        elif url.endswith(".tar.gz") or url.endswith(".tgz"):
-            await self.untar(source, "-z")
-        elif url.endswith(".tar.bz2"):
-            await self.untar(source, "-j")
-        elif url.endswith(".tar.xz") or url.endswith(".txz"):
-            await self.untar(source, "-J")
-        elif url.endswith(".zip"):
-            await self.unzip(source)
-        elif url.endswith(".AppImage"):
-            await self.appimage(source)
-        else:
-            raise ValueError(f"Unknown archive format: {url}")
+            return
+
+        extractor = await get_extractor(url, driver=self.driver)
+        await extractor.extract(
+            archive=source, target_directory=await self.package_directory()
+        )
 
     async def find_in_package_directory(
         self,
@@ -98,46 +61,64 @@ class ArchivePackage(ManualPackage, ABC):
     ) -> Path:
         for relative_path in paths:
             candidate = await self.package_directory() / Path(*relative_path) / name
-            if require_executable:
-                candidate_ok = await self.driver.is_executable(candidate)
-            else:
-                candidate_ok = await self.driver.is_file(candidate)
-            if candidate_ok:
-                return candidate
+            if not await self.driver.is_file(candidate):
+                continue
+            if require_executable and not await self.driver.is_executable(candidate):
+                continue
+            return candidate
         raise ValueError(
             f"Cannot find {target_desc} '{name}' in {await self.package_directory()}."
         )
 
     async def binary_path(self, binary: str) -> Path:
         return await self.find_in_package_directory(
-            paths=[[], ["bin"]],
+            paths=[[], ["bin"], *([bp] for bp in self.binary_paths)],
             name=binary,
             require_executable=True,
             target_desc="binary",
         )
 
-    async def app_path(self, name: str) -> Path:
+    async def freedesktop_app_path(self, name: str) -> Path:
         return await self.find_in_package_directory(
             paths=[["share", "applications"]],
             name=f"{name}.desktop",
             target_desc="application",
         )
 
-    async def icon_paths(self, name: str) -> Iterable[Path]:
-        return await self.driver.find(
-            await self.package_directory(), name=[f"{name}.svg", f"{name}.png"]
+    async def macos_app_path(self, name: str) -> Path:
+        dir_name = f"{name}.app"
+        candidates = await self.driver.find(
+            await self.package_directory(), name=dir_name, file_type="d"
         )
+        if candidates:
+            return candidates[0]
+        else:
+            raise ValueError(
+                f"Cannot find application '{name}' in {await self.package_directory()}."
+            )
+
+    @staticmethod
+    def with_extensions(name: str, extensions: list[str]) -> list[str]:
+        if any(name.endswith(f".{ext}") for ext in extensions):
+            return [name]
+        return [f"{name}.{ext}" for ext in extensions]
+
+    async def icon_paths(self, name: str) -> Iterable[Path]:
+        names = self.with_extensions(name, ICON_EXTENSIONS)
+        return await self.driver.find(await self.package_directory(), name=names)
 
     async def font_path(self, name: str) -> Path:
-        candidate = await self.package_directory() / name
-        if await self.driver.is_file(candidate):
-            return candidate
-        raise ValueError(
-            f"Cannot find font '{name}' in {await self.package_directory()}."
-        )
+        names = self.with_extensions(name, FONT_EXTENSIONS)
+        candidates = await self.driver.find(await self.package_directory(), name=names)
+        if candidates:
+            return candidates[0]
+        else:
+            raise ValueError(
+                f"Cannot find font '{name}' in {await self.package_directory()}."
+            )
 
     async def install_appimage(self, tracker: Tracker) -> None:
-        app_dir = await self.package_directory() / "squashfs-root"
+        app_dir = await self.package_directory()
         app_run = app_dir / "AppRun"
         if not await self.driver.is_executable(app_run):
             raise ValueError("AppImage does not have an executable named 'AppRun'.")
@@ -145,7 +126,7 @@ class ArchivePackage(ManualPackage, ABC):
         binary_name = self.pathname
         binary_target = await self.local() / "bin" / binary_name
         await self.install_binary_wrapper(app_run, binary_target)
-        tracker.track(binary_target)
+        tracker.track(binary_target, root=self.root)
 
         desktop_files = await self.driver.find(app_dir, name="*.desktop", maxdepth=1)
         if not desktop_files:
@@ -157,17 +138,16 @@ class ArchivePackage(ManualPackage, ABC):
         desktop_entry = DesktopEntry.from_string(
             await self.driver.read_file(desktop_file)
         )
-        desktop_entry.exec = " ".join(
-            [binary_name, desktop_entry.exec.split(" ", 1)[1]]
-        )
+        [_, *args] = desktop_entry.exec.split(" ")
+        desktop_entry.exec = " ".join([binary_name, *args])
 
         target_desktop_file = await self.application_path() / desktop_file.name
         await self.driver.write_file(target_desktop_file, desktop_entry.to_string())
-        tracker.track(target_desktop_file)
+        tracker.track(target_desktop_file, root=self.root)
         if desktop_entry.icon:
             await self.install_icon(desktop_entry.icon, tracker)
 
-    async def install_tracked(self, *, tracker: Tracker):
+    async def install(self, *, tracker: Tracker):
         url = await self.archive_url()
         async with self.driver.tempfile() as archive_path:
             await self.driver.run(
@@ -182,12 +162,12 @@ class ArchivePackage(ManualPackage, ABC):
 
             package_directory = await self.package_directory()
             await self.driver.makedirs(package_directory)
-            tracker.track(package_directory.parent)
-            tracker.track(package_directory)
+            tracker.track(package_directory.parent, root=self.root)
+            tracker.track(package_directory, root=self.root)
 
             await self.extract(url, archive_path)
 
         if url.endswith(".AppImage"):
             await self.install_appimage(tracker)
 
-        await super().install_tracked(tracker=tracker)
+        await super().install(tracker=tracker)
