@@ -2,20 +2,19 @@ from typing import Optional
 
 from pydantic import Field
 
-from ..utils import RunArg, async_cached, run_output
+from ..tracker import Tracker
+from ..utils import RunArg, async_cached, repo_version
 from .destination import Destination
-from .tracked import Tracked, Tracker
 
 
-class Clone(Destination, Tracked):
+class Clone(Destination):
     repo: str = Field(..., alias="clone")
 
-    @property
-    def name(self) -> str:
+    def derive_name(self) -> str:
         return self.repo
 
     async def directory_exists(self) -> bool:
-        return await self.driver.run_ok("test", "-d", await self.destination())
+        return await self.driver.is_dir(await self.destination())
 
     @async_cached
     async def git_args(self) -> list[RunArg]:
@@ -36,9 +35,9 @@ class Clone(Destination, Tracked):
         return f"https://github.com/{self.repo}.git"
 
     async def get_remote_version(self) -> str:
-        return (await run_output("git", "ls-remote", self.remote, "HEAD")).split()[0]
+        return await repo_version(self.remote)
 
-    async def install_tracked(self, *, tracker: Tracker) -> None:
+    async def install(self, *, tracker: Tracker) -> None:
         destination = await self.destination()
 
         await self.driver.makedirs(destination.parent)
@@ -56,6 +55,6 @@ class Clone(Destination, Tracked):
         await self.run_git("switch", default_branch)
         await self.run_git("reset", "--hard", f"origin/{default_branch}")
 
-        tracker.track(destination)
+        tracker.track(destination, root=self.root)
 
-        await super().install_tracked(tracker=tracker)
+        await super().install(tracker=tracker)
