@@ -2,19 +2,20 @@ import hashlib
 from pathlib import Path
 from typing import Iterator, Optional
 
-from pydantic import Field, validator
+from pydantic import Field, field_validator
 
+from ..tracker import Tracker
 from ..utils import allow_singular
 from .destination import Destination
 from .manual_version import ManualVersion
-from .tracked import Tracked, Tracker
 
 
-class Links(ManualVersion, Destination, Tracked):
+class Links(ManualVersion, Destination):
     source: Path = Field(..., alias="links")
 
-    @validator("source")
-    def source_absolute(cls, value):  # pylint:disable=no-self-argument
+    @field_validator("source")
+    @classmethod
+    def source_absolute(cls, value):
         return Path(value).absolute()
 
     dot: bool = False
@@ -22,8 +23,7 @@ class Links(ManualVersion, Destination, Tracked):
     only: Optional[list[str]] = None
     only_val = allow_singular("only")
 
-    @property
-    def name(self) -> str:
+    def derive_name(self) -> str:
         return f"links-{self.source}-{self.destination_}-{self.dot}-{self.root}"
 
     def all_paths(self) -> Iterator[Path]:
@@ -47,7 +47,7 @@ class Links(ManualVersion, Destination, Tracked):
             m.update(str(path).encode())
         return m.hexdigest()
 
-    async def install_tracked(self, *, tracker: Tracker) -> None:
+    async def install(self, *, tracker: Tracker) -> None:
         destination = await self.destination()
 
         for path in self.paths():
@@ -60,4 +60,4 @@ class Links(ManualVersion, Destination, Tracked):
             await self.driver.link(path, target)
             tracker.track(target, root=self.root)
 
-        await super().install_tracked(tracker=tracker)
+        await super().install(tracker=tracker)
