@@ -20,3 +20,10 @@ class Root(Package, ABC):
             return Path("/usr/local")
         else:
             return await self.driver.local()
+
+    @async_cached
+    async def macos_applications(self) -> Path:
+        if self.root:
+            return Path("/Applications")
+        else:
+            return (await self.driver.home()) / "Applications"
