@@ -1,27 +1,18 @@
 import subprocess
 from functools import wraps
 from pathlib import Path
-from typing import (
-    Any,
-    Callable,
-    Coroutine,
-    Iterable,
-    Optional,
-    TypeVar,
-    Union,
-    overload,
-)
+from typing import Any, Callable, Coroutine, Iterable, Optional, TypeVar, overload
 
 import requests
 import trio
-from pydantic import validator
+from pydantic import field_validator
 
 T = TypeVar("T")
 U = TypeVar("U")
 V = TypeVar("V")
 
 
-Some = Optional[Union[T, list[T]]]
+Some = Optional[T | list[T]]
 
 
 def unsome_(x: Some[T]) -> Optional[list[T]]:
@@ -37,11 +28,11 @@ def unsome(x: Some[T]) -> list[T]:
 
 
 def allow_singular(field: str) -> Any:
-    return validator(field, pre=True, allow_reuse=True)(lambda cls, x: unsome_(x))
+    return field_validator(field, mode="before")(lambda cls, x: unsome_(x))
 
 
 def allow_singular_none(field: str) -> Any:
-    return validator(field, pre=True, allow_reuse=True)(lambda cls, x: unsome(x))
+    return field_validator(field, mode="before")(lambda cls, x: unsome(x))
 
 
 def intercalate(delimiter: T, items: Iterable[Iterable[T]]) -> list[T]:
@@ -56,7 +47,7 @@ def intercalate(delimiter: T, items: Iterable[Iterable[T]]) -> list[T]:
     return result
 
 
-RunArg = Union[str, Path]
+RunArg = str | Path
 
 
 async def run(*args: RunArg) -> subprocess.CompletedProcess:
@@ -89,6 +80,15 @@ def url_version(url: str) -> str:
     return head_response.headers["etag"]
 
 
+GIT_PREFIX = "git+"
+
+
+async def repo_version(repo: str) -> str:
+    if repo.startswith(GIT_PREFIX):
+        repo = repo.removeprefix(GIT_PREFIX)
+    return (await run_output("git", "ls-remote", repo, "HEAD")).split()[0]
+
+
 def raise_(exception: BaseException) -> Any:
     raise exception
 
@@ -97,18 +97,17 @@ AsyncRet = Coroutine[Any, Any, T]
 
 
 @overload
-def async_cached(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]:
-    ...
+def async_cached(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]: ...
 
 
 @overload
-def async_cached(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]:
-    ...
+def async_cached(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]: ...
 
 
 @overload
-def async_cached(fn: Callable[[U, V], AsyncRet[T]]) -> Callable[[U, V], AsyncRet[T]]:
-    ...
+def async_cached(
+    fn: Callable[[U, V], AsyncRet[T]]
+) -> Callable[[U, V], AsyncRet[T]]: ...
 
 
 def async_cached(fn: Callable[..., AsyncRet[T]]) -> Callable[..., AsyncRet[T]]:
@@ -116,20 +115,17 @@ def async_cached(fn: Callable[..., AsyncRet[T]]) -> Callable[..., AsyncRet[T]]:
 
 
 @overload
-def async_cached_lock(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]:
-    ...
+def async_cached_lock(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]: ...
 
 
 @overload
-def async_cached_lock(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]:
-    ...
+def async_cached_lock(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]: ...
 
 
 @overload
 def async_cached_lock(
     fn: Callable[[U, V], AsyncRet[T]]
-) -> Callable[[U, V], AsyncRet[T]]:
-    ...
+) -> Callable[[U, V], AsyncRet[T]]: ...
 
 
 def async_cached_lock(fn: Callable[..., AsyncRet[T]]) -> Callable[..., AsyncRet[T]]:
